package cn.com.qmth.examcloud.core.basic.api.controller;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import cn.com.qmth.examcloud.commons.base.util.ErrorMsg;
import cn.com.qmth.examcloud.commons.base.util.excel.ExcelError;
import cn.com.qmth.examcloud.commons.web.security.entity.AccessUser;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.dao.SpecialtyRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.Specialty;
import cn.com.qmth.examcloud.core.basic.service.bean.SpecialtyDto;
import cn.com.qmth.examcloud.core.basic.service.impl.ExportService;
import cn.com.qmth.examcloud.core.basic.service.impl.SpecialtyService;
import io.swagger.annotations.ApiOperation;

/**
 * @Description: 专业服务API
 * @author weiwenhai
 * @Date  2017年8月7号
 */
@RestController
@RequestMapping("${$rmp.ctr.basic}/specialty")
public class SpecialtyController extends ControllerSupport{

	@Autowired
	SpecialtyService specialtyService;
	
	@Autowired
	SpecialtyRepo specialtyRepo;
	
	/**
	 * 带分页查询
	 * @param specialty
	 * @param curPage
	 * @param pageSize
	 * @param request
	 * @return
	 */
	@ApiOperation(value="查询专业带分页", notes="查询专业带分页")
	@GetMapping("/all/{curPage}/{pageSize}")
	public ResponseEntity getAllOrg(@ModelAttribute Specialty specialty,
									@RequestParam(required = false ) Long courseId,
									@PathVariable Integer curPage,
									@PathVariable Integer pageSize,
									HttpServletRequest request){
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if(accessUser != null){
			specialty.setOrgId(accessUser.getRootOrgId());
		}else {
			return new ResponseEntity(HttpStatus.NOT_FOUND);
		}
		return new ResponseEntity(specialtyService.findAll(specialty,courseId, new PageRequest(curPage-1, pageSize)),HttpStatus.OK);
	}

	/**
	 * 按code查询专业
	 * @param orgId
	 * @param code
	 * @return
	 */
	@ApiOperation(value="按code查询专业",notes="按code查询专业")
	@GetMapping
	public ResponseEntity getAll(@RequestParam Long orgId, @RequestParam String code){
		return new ResponseEntity(specialtyRepo.findByOrgIdAndCode(orgId,code),HttpStatus.OK);
	}
	
	/**
	 * 查询专业不带分页
	 * @param specialty
	 * @param request
	 * @return
	 */
	@ApiOperation(value="查询专业不带分页",notes="查询专业不带分页")
	@GetMapping("/all")
	public ResponseEntity getAll(@ModelAttribute Specialty specialty,HttpServletRequest request){
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if(accessUser != null){
			specialty.setOrgId(accessUser.getRootOrgId());
		}
		specialty.setEnable(true);
		return new ResponseEntity(specialtyService.findAll(specialty),HttpStatus.OK);
	}
	
	/**
	 * 新增专业
	 * @param specialty
	 * @param request
	 * @return
	 */
	@ApiOperation(value="新增专业", notes="新增专业")
	@PostMapping("/addSpecialty")
	public ResponseEntity addSpecialty(@RequestBody Specialty specialty,HttpServletRequest request){
		specialty.setCreateTime(new Date());
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if(accessUser != null){
			specialty.setOrgId(accessUser.getRootOrgId());
		}else {
			return new ResponseEntity(HttpStatus.NOT_FOUND);
		}
		try{
			specialtyService.save(specialty);
			return new ResponseEntity(HttpStatus.CREATED);
		} catch (Exception e) {
			 e.printStackTrace();
			 return new ResponseEntity(new ErrorMsg(e.getMessage()), HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}
	
	/**
	 * 更新专业
	 * @param specialty
	 * @param request
	 * @return
	 */
	@ApiOperation(value="更新专业",notes="更新专业")
	@PutMapping("/updateSpecialty")
	public ResponseEntity updateSpecialty(@RequestBody Specialty specialty,HttpServletRequest request){
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if(accessUser != null){
			specialty.setOrgId(accessUser.getRootOrgId());
		}else {
			return new ResponseEntity(HttpStatus.NOT_FOUND);
		}
		try{
			specialtyService.updateSpecialty(specialty);
			return new ResponseEntity(HttpStatus.OK);
		} catch (Exception e) {
			 e.printStackTrace();
			 return new ResponseEntity(new ErrorMsg(e.getMessage()), HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}
	
	/**
	 * 根据ID删除专业
	 * @param ids
	 * @return
	 */
	@ApiOperation(value="根据ID删除专业",notes="根据ID删除专业")
	@DeleteMapping("/{ids}")
	public ResponseEntity deleteSpecialtyIds(@PathVariable String ids){
		specialtyService.deleteSpecialty(ids);
		return new ResponseEntity(HttpStatus.OK);
	}
	
	@ApiOperation(value="根据ID删除专业",notes="根据ID删除专业")
	@DeleteMapping("/delete/{id}")
	public ResponseEntity deleteSpecialtyId(@PathVariable String id){
		specialtyRepo.delete(Long.parseLong(id));
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value="下载导入模板",notes = "下载导入模板")
	@GetMapping("/download")
	public void importFileTemplate(HttpServletResponse response){
		List<SpecialtyDto> list= new ArrayList<SpecialtyDto>();
		ExportService.exportEXCEL("专业导入模板", SpecialtyDto.class, list, response);
	}

	@ApiOperation(value="导入",notes = "导入")
	@PostMapping("/import")
	public ResponseEntity importCourse(HttpServletRequest request,
									   @RequestParam CommonsMultipartFile file){
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		Long orgId = null;
		if(accessUser != null){
			orgId = accessUser.getRootOrgId();
		}else{
			return new ResponseEntity(new ErrorMsg("用户token不存在或已失效"),HttpStatus.NOT_FOUND);
		}
		try {
			List<ExcelError> errors = specialtyService.importSpecialty(orgId,file.getInputStream());
			return new ResponseEntity(errors,HttpStatus.OK);
		} catch (IOException e) {
			e.printStackTrace();
			return new ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}
	
	/**
	 * 导出专业
	 * @param specialty
	 * @param response
	 */
	@ApiOperation(value="导出专业", notes="导出专业")
	@GetMapping("/export")
	public void exportSpecialty(@ModelAttribute Specialty specialty,HttpServletResponse response){
		List<SpecialtyDto> list = new ArrayList<SpecialtyDto>();
		specialtyService.findAll(specialty).forEach(c -> {
			list.add(new SpecialtyDto(c));
		});
		ExportService.exportEXCEL("专业列表", SpecialtyDto.class, list, response);
	}
}
