package cn.com.qmth.examcloud.core.basic.api.provider;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.StudentBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetStudentReq;
import cn.com.qmth.examcloud.core.basic.api.request.InsertOrUpdateStudentReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetStudentResp;
import cn.com.qmth.examcloud.core.basic.api.response.InsertOrUpdateStudentResp;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.StudentRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.Org;
import cn.com.qmth.examcloud.core.basic.dao.entity.Student;
import cn.com.qmth.examcloud.core.basic.service.bean.StudentInfo;
import cn.com.qmth.examcloud.core.basic.service.impl.StudentServiceImpl;
import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping("${$rmp.cloud.basic}" + "student")
public class StudentCloudServiceProvider extends ControllerSupport implements StudentCloudService {

	private static final long serialVersionUID = -571041158847398070L;

	@Autowired
	OrgRepo orgRepo;

	@Autowired
	UserRepo userRepo;

	@Autowired
	StudentServiceImpl studentService;

	@Autowired
	StudentRepo studentRepo;

	@ApiOperation(value = "插入或更新学生")
	@PostMapping("insertOrUpdateStudent")
	@Override
	public InsertOrUpdateStudentResp insertOrUpdateStudent(
			@RequestBody InsertOrUpdateStudentReq req) {

		StudentInfo studentInfo = new StudentInfo();
		studentInfo.setName(req.getName());
		studentInfo.setIdentityNumber(req.getIdentityNumber());
		studentInfo.setStudentCode(req.getStudentCode());
		studentInfo.setRootOrgId(req.getRootOrgId());

		studentInfo.setOrgCode(req.getOrgCode());
		studentInfo.setOrgName(req.getOrgName());

		studentInfo.setPhoneNumber(req.getPhoneNumber());
		studentInfo.setPhotoPath(req.getPhotoPath());
		studentInfo.setRemark(req.getRemark());

		Student student = studentService.insertOrUpdateStudent(studentInfo);
		InsertOrUpdateStudentResp resp = new InsertOrUpdateStudentResp();

		resp.setRootOrgId(student.getRootOrgId());

		Org org = orgRepo.findOne(student.getOrgId());
		resp.setOrgId(org.getId());
		resp.setOrgName(org.getName());
		return resp;
	}

	@ApiOperation(value = "查询学生")
	@PostMapping("getStudent")
	@Override
	public GetStudentResp getStudent(GetStudentReq req) {
		Long rootOrgId = req.getRootOrgId();
		String studentCode = req.getStudentCode();
		String identityNumber = req.getIdentityNumber();

		if (null == rootOrgId) {
			throw new StatusException("B-150000", "rootOrgId is null");
		}
		if (StringUtils.isBlank(studentCode)) {
			throw new StatusException("B-150003", "studentCode is null");
		}
		if (StringUtils.isBlank(identityNumber)) {
			throw new StatusException("B-150004", "identityNumber is null");
		}

		Org rootOrg = orgRepo.findOne(rootOrgId);
		if (null == rootOrg) {
			throw new StatusException("B-150001", "机构不存在");
		}
		if (!rootOrg.getParentId().equals(0L)) {
			throw new StatusException("B-150002", "机构错误");
		}

		Student student = studentRepo.findByIdentityNumberAndStudentCodeAndRootOrgId(identityNumber,
				studentCode, rootOrgId);

		if (null == student) {
			throw new StatusException("B-150005", "学生不存在");
		}

		Org org = orgRepo.findOne(student.getOrgId());

		StudentBean studentBean = new StudentBean();
		studentBean.setId(student.getId());
		studentBean.setName(student.getName());
		studentBean.setIdentityNumber(student.getIdentityNumber());
		studentBean.setStudentCode(student.getStudentCode());
		studentBean.setRootOrgId(student.getRootOrgId());
		studentBean.setOrgCode(org.getCode());
		studentBean.setOrgName(org.getName());
		studentBean.setPhoneNumber(student.getPhoneNumber());
		studentBean.setPhotoPath(student.getPhotoPath());
		studentBean.setRemark(student.getRemark());

		GetStudentResp resp = new GetStudentResp();
		resp.setStudentInfo(studentBean);
		return resp;
	}

}
