package cn.com.qmth.examcloud.core.basic.service.impl;

import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.esotericsoftware.minlog.Log;
import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.ByteUtil;
import cn.com.qmth.examcloud.commons.base.util.PropertiesUtil;
import cn.com.qmth.examcloud.commons.base.util.SHA256;
import cn.com.qmth.examcloud.commons.base.util.StringUtil;
import cn.com.qmth.examcloud.commons.base.util.UUID;
import cn.com.qmth.examcloud.commons.web.redis.RedisClient;
import cn.com.qmth.examcloud.commons.web.security.bean.Role;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.security.enums.RoleMeta;
import cn.com.qmth.examcloud.core.basic.base.constants.PropKeys;
import cn.com.qmth.examcloud.core.basic.base.enums.AccountType;
import cn.com.qmth.examcloud.core.basic.base.enums.UserType;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.RoleRepo;
import cn.com.qmth.examcloud.core.basic.dao.StudentRepo;
import cn.com.qmth.examcloud.core.basic.dao.ThirdPartyAccessRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.Org;
import cn.com.qmth.examcloud.core.basic.dao.entity.Student;
import cn.com.qmth.examcloud.core.basic.dao.entity.ThirdPartyAccessEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.ThirdPartyAccessPK;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserRole;
import cn.com.qmth.examcloud.core.basic.service.AuthService;
import cn.com.qmth.examcloud.core.basic.service.UserService;
import cn.com.qmth.examcloud.core.basic.service.bean.LoginInfo;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年5月25日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Service
public class AuthServiceImpl implements AuthService {

	@Autowired
	RedisClient redisClient;

	@Autowired
	UserService userService;

	@Autowired
	UserRepo userRepo;

	@Autowired
	OrgRepo orgRepo;

	@Autowired
	RoleRepo roleRepo;

	@Autowired
	StudentRepo studentRepo;

	@Autowired
	ThirdPartyAccessRepo thirdPartyAccessRepo;

	@Override
	public User login(LoginInfo loginInfo) {

		String rootOrgId = loginInfo.getRootOrgId();
		Org rootOrg = null;
		if (StringUtils.isBlank(rootOrgId)) {
			if (StringUtils.isBlank(loginInfo.getDomain())) {
				throw new StatusException("B-001001", "domain,rootOrgId 必须有一个不为空");
			}

			try {
				rootOrg = orgRepo.findFirstByParentIdAndCode(0L, loginInfo.getDomain());
				rootOrgId = String.valueOf(rootOrg.getId());
			} catch (Exception e) {
				throw new StatusException("B-001002", "机构不存在", e);
			}

		} else {
			rootOrg = orgRepo.findOne(Long.valueOf(rootOrgId));
			if (null == rootOrg) {
				throw new StatusException("B-001002", "机构不存在");
			}
		}

		String accountType = loginInfo.getAccountType();
		String accountValue = loginInfo.getAccountValue();
		String password = loginInfo.getPassword();

		UserEntity userEntity = null;
		Student student = null;
		User user = new User();

		// 常规账户登录
		if (AccountType.COMMON_LOGIN_NAME.getCode().equals(accountType)) {
			user.setUserType(UserType.COMMON.getCode());
			userEntity = userService.getUser(Long.parseLong(rootOrgId), accountValue);
			if (null == userEntity) {
				throw new StatusException("B-001004", "用户不存在");
			}
			user.setDisplayName(userEntity.getLoginName());
		} else {
			// 学生学号登录
			if (AccountType.STUDENT_CODE.getCode().equals(accountType)) {
				student = studentRepo.findByStudentCodeAndRootOrgId(accountValue, rootOrg.getId());
			}
			// 学生身份证号登录
			else if (AccountType.STUDENT_IDENTITY_NUMBER.getCode().equals(accountType)) {
				student = studentRepo.findByIdentityNumberAndRootOrgId(accountValue,
						rootOrg.getId());
			}

			if (null == student) {
				throw new StatusException("B-001110", "学生不存在");
			}
			
			user.setUserType(UserType.STUDENT.getCode());
			user.setStudentId(student.getId());
			user.setStudentCode(student.getStudentCode());
			user.setIdentityNumber(student.getIdentityNumber());

			userEntity = userRepo.findOne(student.getUser().getId());
			if (null == userEntity) {
				throw new StatusException("B-001004", "用户不存在");
			}
			user.setDisplayName(userEntity.getName());
			List<UserRole> roleList = Lists.newArrayList();
			UserRole userRole = new UserRole();
			userRole.setRoleCode(RoleMeta.STUDENT.name());
			roleList.add(userRole);
			userEntity.setUserRoles(roleList);
		}
		
		String rightPassword = userEntity.getPassword();
		if (!rightPassword.equals(password)) {
			throw new StatusException("B-001003", "密码错误");
		}

		user.setUserId(userEntity.getId());
		
		user.setRootOrgId(userEntity.getRootOrgId());
		user.setRootOrgName(rootOrg.getName());
		user.setOrgId(userEntity.getOrgId());
		if (null != user.getOrgId()) {
			Org org = orgRepo.findOne(user.getOrgId());
			user.setOrgName(org.getName());
		}

		List<UserRole> userRoles = userEntity.getUserRoles();
		List<Role> roleList = getRoles(userRoles);
		user.setRoleList(roleList);

		setSession(user);

		return user;
	}

	/**
	 * 构建key
	 *
	 * @author WANGWEI
	 * @param userType
	 * @param rootOrgId
	 * @param userId
	 * @return
	 */
	private String buildUserKey(String userType, Long rootOrgId, Long userId) {
		String key = StringUtils.join("U_", userType, "_", String.valueOf(rootOrgId), "_",
				String.valueOf(userId));
		return key;
	}

	@Override
	public void logout(User user) {
		redisClient.delete(user.getKey());
	}

	@Override
	public User getLoginUser(String key, String token) {
		int sessionTimeout = PropertiesUtil.getInt(PropKeys.SESSION_TIMEOUT, 3600);
		User user = redisClient.get(key, User.class, sessionTimeout);
		if (null == user) {
			throw new StatusException("P-001012", "未登录");
		}

		if (!user.getToken().equals(token)) {
			throw new StatusException("P-001012", "token错误");
		}

		return user;
	}

	@Override
	public User thirdPartyAccess(Long rootOrgId, String loginName, String appId, String timestamp,
			String token) throws StatusException {

		Org rootOrg = orgRepo.findOne(rootOrgId);
		if (null == rootOrg) {
			throw new StatusException("B-001002", "机构不存在");
		}

		ThirdPartyAccessEntity thirdPartyAccess = thirdPartyAccessRepo
				.findOne(new ThirdPartyAccessPK(rootOrgId, appId));

		if (null == thirdPartyAccess) {
			throw new StatusException("B-001201", "第三方系统接入信息未配置");
		}

		long timestampLong = 0L;
		try {
			timestampLong = Long.parseLong(timestamp);
		} catch (Exception e) {
			throw new StatusException("B-001202", "timestamp错误");
		}

		long currentTimeMillis = System.currentTimeMillis();
		Log.debug("currentTimeMillis = " + currentTimeMillis);
		if (Math.abs(currentTimeMillis - timestampLong) > thirdPartyAccess.getTimeRange()) {
			throw new StatusException("B-001203", "timestamp超出时间差范围");
		}

		String secretKey = thirdPartyAccess.getSecretKey();
		String joinStr = StringUtil.join(loginName, rootOrgId, appId, timestamp, secretKey);
		byte[] bytes = SHA256.encode(joinStr);
		String hexAscii = ByteUtil.toHexAscii(bytes);

		if (!hexAscii.toLowerCase(Locale.US).equals(token.toLowerCase(Locale.US))) {
			throw new StatusException("B-001204", "token校验失败");
		}

		UserEntity userEntity = userRepo.findByRootOrgIdAndLoginName(rootOrgId, loginName);
		if (userEntity == null) {
			throw new StatusException("B-001205", "用户不存在");
		}

		User user = new User();
		user.setUserType(UserType.COMMON.getCode());
		user.setUserId(userEntity.getId());
		user.setDisplayName(userEntity.getLoginName());
		user.setRootOrgId(userEntity.getRootOrgId());
		user.setRootOrgName(rootOrg.getName());
		user.setOrgId(userEntity.getOrgId());
		if (null != user.getOrgId()) {
			Org org = orgRepo.findOne(user.getOrgId());
			user.setOrgName(org.getName());
		}

		List<UserRole> userRoles = userEntity.getUserRoles();
		List<Role> roleList = getRoles(userRoles);
		user.setRoleList(roleList);

		setSession(user);

		return user;
	}

	/**
	 * 设置session
	 *
	 * @author WANGWEI
	 * @param user
	 */
	private void setSession(User user) {
		user.setTokenCreationTime(new Date());
		user.setToken(UUID.randomUUID());

		String key = buildUserKey(user.getUserType(), user.getRootOrgId(), user.getUserId());
		user.setKey(key);
		user.setUserToken(key + ":" + user.getToken());

		int sessionTimeout = PropertiesUtil.getInt(PropKeys.SESSION_TIMEOUT, 3600);
		Integer sessionTimeoutCache = redisClient.get("$_SESSION_TIMEOUT", Integer.class);
		if (null == sessionTimeoutCache || (int) sessionTimeoutCache != sessionTimeout) {
			redisClient.set("$_SESSION_TIMEOUT", sessionTimeout);
		}

		redisClient.set(key, user, sessionTimeout);
	}

	/**
	 * 获取角色集合
	 *
	 * @author WANGWEI
	 * @param userRoles
	 * @return
	 */
	private List<Role> getRoles(List<UserRole> userRoles) {
		List<Role> roleList = Lists.newArrayList();
		if (CollectionUtils.isNotEmpty(userRoles)) {
			for (UserRole cur : userRoles) {
				String roleCode = cur.getRoleCode();
				cn.com.qmth.examcloud.core.basic.dao.entity.RoleEntity roleEntity = roleRepo
						.findByCode(roleCode);
				if (null == roleEntity) {
					throw new StatusException("B-002002",
							"role code is wrong. roleCode=" + roleCode);
				}
				Role role = new Role(roleEntity.getId(), roleEntity.getCode(),
						roleEntity.getName());
				roleList.add(role);
			}
		}
		return roleList;
	}

}
